/*
 *      $Source: /usr/local/libdata/cvs/simons/petidomo/src/htmlconf/handlelistconfpage.c,v $
 *      $Revision: 1.7 $
 *      $Date: 1998/06/01 17:19:27 $
 *
 *      Copyright (C) 1996 by Peter Simons, Germany
 *      All rights reserved.
 */

#include <errno.h>

#include <cgi.h>
#include <text.h>
#include <petidomo.h>

extern cgi_info      ci;
extern form_entry *  form_parameter;

void printMainPage(void);
void handleBlanks(char **);

void
handleListConfPage(void)
{
    const struct PD_Config * MasterConfig = getMasterConfig();
    FILE *     fh;
    char *     filename;
    char *     listname = parmval(form_parameter, "listname");
    char *     password = parmval(form_parameter, "password");
    char *     Hostname = parmval(form_parameter, "Hostname");
    char *     adminpassword = parmval(form_parameter, "adminpassword");
    char *     adminpassword2 = parmval(form_parameter, "adminpassword2");
    char *     allowaliensub = parmval(form_parameter, "allowaliensub");
    char *     allowmembers = parmval(form_parameter, "allowmembers");
    char *     allowpubsub = parmval(form_parameter, "allowpubsub");
    char *     archivepath = parmval(form_parameter, "archivepath");
    char *     listtype = parmval(form_parameter, "listtype");
    char *     postingfilter = parmval(form_parameter, "postingfilter");
    char *     postingpassword = parmval(form_parameter, "postingpassword");
    char *     postingpassword2 = parmval(form_parameter, "postingpassword2");
    char *     replyto = parmval(form_parameter, "replyto");
    char *     showonindex = parmval(form_parameter, "showonindex");

    /* Do consistency checks. */

    if (isValidAdminPassword(password, listname) == FALSE) {
	puts("<H1 ALIGN=center>Access denied!</H1>");
	puts("The password you entered does not authorize you to edit the");
	puts("mailing list config file, I am afraid.");
	return;
    }

    if (strcasecmp(adminpassword, adminpassword2) != 0) {
	puts("<H3>ERROR</H1>");
	puts("You mis-typed the administrator password. Please use the \n");
	puts("\"previous page\" function of your browser to edit the \n");
	puts("entries in the form again.<P>");
	return;
    }

    if (strcasecmp(postingpassword, postingpassword2) != 0) {
	puts("<H3>ERROR</H1>");
	puts("You mis-typed the posting password. Please use the \n");
	puts("\"previous page\" function of your browser to edit the \n");
	puts("entries in the form again.<P>");
	return;
    }

    handleBlanks(&adminpassword);
    handleBlanks(&postingpassword);
    handleBlanks(&postingfilter);

    /* Write the new config file back. */

    filename = text_easy_sprintf("lists/%s/config", listname);
    fh = fopen(filename, "w");
    if (fh == NULL) {
	puts("<H3>ERROR</H1>");
	printf("Opening the config file \"%s\" failed with the following error:<P>\n",
	       filename);
	printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	puts("Please contact the administrator with this problem.<P>\n");
	return;
    }
    fprintf(fh, "#\n");
    fprintf(fh, "# Config file for mailing list \"%s\"\n", listname);
    fprintf(fh, "# generated automatically by petidomoconf\n");
    fprintf(fh, "#\n");
    fprintf(fh, "ListType\t\t%s\n", listtype);
    fprintf(fh, "AllowPublicSubscription\t%s\n", allowpubsub);
    fprintf(fh, "AllowAlienSubscription\t%s\n", allowaliensub);
    fprintf(fh, "AllowMembersCommand\t%s\n", allowmembers);
    fprintf(fh, "ShowOnIndex\t\t%s\n", showonindex);
    if (!strcasecmp(MasterConfig->fqdn, Hostname))
      fprintf(fh, "#");
    fprintf(fh, "Hostname\t\t%s\n", Hostname);
    if (*adminpassword == '\0')
      fprintf(fh, "#AdminPasswort\t\t\"open sesame\"\n");
    else
      fprintf(fh, "AdminPassword\t\t%s\n", adminpassword);
    if (*postingpassword == '\0')
      fprintf(fh, "#PostingPasswort\t\"lemme post\"\n");
    else
      fprintf(fh, "PostingPassword\t\t%s\n", postingpassword);
    if (*replyto == '\0')
      fprintf(fh, "#ReplyTo\t\tnone\n");
    else
      fprintf(fh, "ReplyTo\t\t%s\n", replyto);
    if (*postingfilter == '\0')
      fprintf(fh, "#PostingFilter\t\t\"/bin/cat\"\n");
    else
      fprintf(fh, "PostingFilter\t\t%s\n", postingfilter);
    if (*archivepath == '\0')
      fprintf(fh, "#Archive\t\t\"log\"\n");
    else
      fprintf(fh, "Archive\t\t%s\n", archivepath);
    fclose(fh);

    /* If all was successful, show the main page again. */

    printMainPage();
    puts("<H3>Success</H1>");
    puts("The new configuration has been set successfully.<P>");
}
