/*
 *      $Source: /usr/local/libdata/cvs/simons/petidomo/src/htmlconf/handleremlist.c,v $
 *      $Revision: 1.9 $
 *      $Date: 1998/08/14 22:54:39 $
 *
 *      Copyright (C) 1997 by Peter Simons, Germany
 *      All rights reserved.
 */

#include <unistd.h>
#include <errno.h>

#include <text.h>
#include <cgi.h>
#include <petidomo.h>

extern cgi_info      ci;
extern form_entry *  form_parameter;

void printMainPage(void);
bool isValidListName(const char *);
int  removeMailinglistAlias(const char *);

void
handleRemoveListPage(void)
{
    char *     listname = parmval(form_parameter, "listname");
    char *     password = parmval(form_parameter, "password");
    char *     filename;
    pid_t      child_pid;
    int        rc;

    /* Do consistency checks. */

    if (listname == NULL || *listname == '\0' ||
	password == NULL || *password == '\0') {
	printMainPage();
	return;
    }

    if (isValidListName(listname) == FALSE) {
	puts("<H3>ERROR</H1>");
	puts("Something went wrong. The listname you chose is not valid.\n");
	puts("This is probably an internal error in the cgi manager.\n");
	return;
    }

    if (isValidAdminPassword(password, listname) == FALSE) {
	puts("<H1 ALIGN=center>Access denied!</H1>");
	puts("The password you entered does not authorize you to delete this");
	puts("mailing list, I am afraid.");
	return;
    }

    /* Remove the mailing list directory. */

    filename = text_easy_sprintf("lists/%s", listname);
    child_pid = fork();
    switch(child_pid) {
      case 0:
          /* Child */
          execl("/bin/rm", "rm", "-rf", filename, NULL);
          return;
      case -1:
          /* Error */
	  puts("<H3>ERROR</H1>");
	  puts("An attempt to fork() failed:<P>\n");
	  printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	  puts("Please contact the administrator with this problem.<P>\n");
          syslog(LOG_ERR, "Couldn't fork: %m");
          return;
      default:
	  ;
          /* everything is fine */
    }

    /* Remove the aliases for the list. */

    rc = removeMailinglistAlias(listname);
    if (rc != 0) {
	printMainPage();
	puts("<H3>ERROR</H1>");
	puts("The mailing list was removed, but removing the required aliases from\n");
	puts("\"/etc/aliases\" failed with the following error:<P>\n");
	printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	puts("The mailing list aliases have to be removed manually.\n");
	puts("Please contact the administrator with this problem.<P>\n");
	return;
    }

    /* If all was successful, show the main page again. */

    printMainPage();
    puts("<H3>Success</H1>");
    puts("The mailing list has been removed successfully.<P>");
}
